import csv
import glob
import os
import pandas as pd
from typing import Dict, List, Tuple

from study1.s1_config import TRAINING_EFFECT_ORDER_RANGE, INPUT_PATH


def get_question_type(block_type, question_nr, question_note, question_type_lookup: Dict[str, List[Tuple[int, int]]]) -> str:
    """
    Looks for the question type in the given question description.

    :param question_desc:
    :param question_type_lookup:
    :return:
    """
    question = (block_type, question_nr)
    for question_type, questions in question_type_lookup.items():
        if question_note in questions:
            return question_type

    for question_type, questions in question_type_lookup.items():
        if question in questions:
            return question_type

    if question_note != "":
        return question_note

    return ""


def get_block_order() -> Dict[int, Dict[int, int]]:
    output = {}

    # Load the information from each answer file
    for dic in [d[0] for d in os.walk(INPUT_PATH)]:
        for fp in glob.glob(f"{dic}/AnswerLog-*.csv"):
            file_name = fp.split('\\')[-1]
            file_nr = int(file_name.split('-')[1])

            last_block = None
            block_index = 1
            blocks = {}
            # Read each file
            with open(fp, newline="") as csv_file:
                reader = csv.reader(csv_file, delimiter=';')
                for i, row in enumerate(reader):
                    if i == 0:
                        continue

                    b = int(row[3])
                    bt = b % 5 if b > 0 else "T"
                    b = b if b > 0 else -b

                    if bt == "T":
                        continue

                    if last_block != b:
                        blocks[b] = block_index
                        block_index += 1
                        last_block = b

            output[file_nr] = blocks

    return output


def convert_order_values(data_frame: pd.DataFrame) -> pd.DataFrame:
    def order_change(row):
        order = row["order"]
        if 1 <= order <= TRAINING_EFFECT_ORDER_RANGE:
            return "Start"
        if TRAINING_EFFECT_ORDER_RANGE < order <= TRAINING_EFFECT_ORDER_RANGE * 2:
            return "Middle-1"
        if TRAINING_EFFECT_ORDER_RANGE * 2 < order <= TRAINING_EFFECT_ORDER_RANGE * 3:
            return "Middle-2"
        elif TRAINING_EFFECT_ORDER_RANGE * 3 < order <= 20:
            return "End"

    data_frame["orderGrouping"] = data_frame.apply(lambda row: order_change(row), axis=1)
    return data_frame

