from collections import namedtuple
from config import IMAGE_OUTPUT_PATH, PRELIMINARY_CHECK_PATH, NORMALITY_IMAGES_PATH, NON_PARAMETRIC_PATH, ANOVA_PATH, QUESTIONNAIRE_PATH

STUDY_INPUT_PATH = "../07 - Logged data (Study 2)"
DATA_SET_INPUT_PATH = "../05 - Line chart data (Study2)"
QUESTIONNAIRE_INPUT_PATH = "../07 - Logged data (Study 2)/numerical_questionnaire.csv"
STUDY_ORDER_INPUT_PATH = "../07 - Logged data (Study 2)/study-order.json"

STUDY_KEY = "S2"

IMAGE_OUTPUT_PATH = f"{IMAGE_OUTPUT_PATH}/{STUDY_KEY}"

PRELIMINARY_CHECK_PATH = f"{PRELIMINARY_CHECK_PATH}/{STUDY_KEY}"
NORMALITY_IMAGES_PATH = f"{NORMALITY_IMAGES_PATH}/{STUDY_KEY}"

NON_PARAMETRIC_PATH = f"{NON_PARAMETRIC_PATH}/{STUDY_KEY}"
ANOVA_PATH = f"{ANOVA_PATH}/{STUDY_KEY}"
QUESTIONNAIRE_PATH = f"{QUESTIONNAIRE_PATH}/{STUDY_KEY}"

QuestionDesc = namedtuple("QuestionDesc", "fileNr, background, backgroundOrder, sessionOrder, blockNr, blockType, visNr")

IGNORE_FILE_NR = [5, 6]  # 4

VIS_MAP = [
    {"block": 1, "vis": 1},
    {"block": 1, "vis": 2},
    {"block": 1, "vis": 3},
    {"block": 1, "vis": 4},
    {"block": 1, "vis": 5},
    {"block": 1, "vis": 6},
    {"block": 1, "vis": 7},
    {"block": 1, "vis": 8},
    {"block": 2, "vis": 1},
    {"block": 2, "vis": 2},
    {"block": 2, "vis": 3},
    {"block": 2, "vis": 4},
    {"block": 2, "vis": 5},
    {"block": 2, "vis": 6},
    {"block": 2, "vis": 7},
    {"block": 2, "vis": 8},
    {"block": 3, "vis": 1},
    {"block": 3, "vis": 2},
    {"block": 3, "vis": 3},
    {"block": 3, "vis": 4},
    {"block": 3, "vis": 5},
    {"block": 3, "vis": 6},
    {"block": 3, "vis": 7},
    {"block": 3, "vis": 8},
]

QUESTION_TYPES = [
    "Retrieve Value",
    "Filter",
    "Compute Derived Value",
    "Extremum",
    "Sort",
    "Determine Range",
    "Cluster",
    "Compare"
]
QUESTION_TYPES_CALCULATED = [
    "Range Values",
    "Filter Correctness",
    "Filter Errors",
    "Sort Errors"
]
QUESTION_MAPPING = {
        "generalQuestion1": "PQ1: The presentation of the visualization in a real environment irritated me.",
        "generalQuestion2": "PQ2: The real-world background did not influenced me.",
        "generalQuestion3": "PQ3: I was able to suppress the real-world background.",
        "backgroundPerception1": "PQ6: How much did the background affect the perception of the lines?",
        "backgroundPerception2": "PQ7: How much did the background affect the perception of the axis labels?",
}
FOCUS_QUESTION_MAPPING = {
        "focusQuestion1": "FQ1: The additional observation task for the background made it harder for me to solve the main task.",
        "focusQuestion2": "FQ2: The additional observation task for the background let me took longer to answer the questions.",
        "focusQuestion3": "FQ3: The additional observation task for the background made my answers for the tasks less precise.",
        "focusQuestion4": "FQ4: The additional observation task for the background needed a great portion of my attention.",
        "focusQuestion5": "FQ5: The additional observation task for the background made it harder to suppress the background.",
}

QUESTION_TYPE_MAP = {
    1: {
        1: QUESTION_TYPES[1],
        2: QUESTION_TYPES[3],
        3: QUESTION_TYPES[0],
        4: QUESTION_TYPES[2]
    },
    2: {
        1: QUESTION_TYPES[4],
        2: QUESTION_TYPES[0],
        3: QUESTION_TYPES[7],
        4: QUESTION_TYPES[5]
    },
    3: {
        1: QUESTION_TYPES[6],
        2: QUESTION_TYPES[2],
        3: QUESTION_TYPES[0],
        4: QUESTION_TYPES[7]
    }
}

COLOR_MAP = [
    "r",
    "b",
    "w",
    "g"
]

TRAINING_EFFECT_ORDER_RANGE = 6

class VisConfigValues:
    QUESTION_TYPES = [
        "Retrieve",
        "Filter",
        "Compute",
        "Extremum",
        "Sort",
        "Range",
        "Cluster",
        "Compare"
    ]
    QUESTION_TYPES_CALCULATED = [
        "Range Values",
        "Filter Correctness",
        "Filter Errors",
        "Sort Errors"
    ]
    QUESTION_TYPES_NUMERIC = [
        QUESTION_TYPES[0],
        QUESTION_TYPES[2],
        QUESTION_TYPES_CALCULATED[0],
        # QUESTION_TYPES_CALCULATED[1],
        QUESTION_TYPES_CALCULATED[2],
        QUESTION_TYPES_CALCULATED[3]
    ]
    QUESTION_TYPES_BOOL = [
        QUESTION_TYPES[1],
        QUESTION_TYPES[3],
        QUESTION_TYPES[4],
        QUESTION_TYPES[5],
        QUESTION_TYPES[6],
        QUESTION_TYPES[7],
    ]

    _YLIM_FACTOR = 1.125
    VIS_VALUES = {
        "EN-yLabels": [i * 1 for i in range(0, 4)],
        "EB-yLabels": [0, 25, 50, 75, 100],
        "T-yLabels": [i * 5 for i in range(0, 5)],
        "TTE-yLabels": [i * 5 for i in range(0, 4)],
        "EN-ylim": (0, 3 * _YLIM_FACTOR),
        "EB-ylim": (0, 1 * _YLIM_FACTOR),
        "T-ylim": (0, 20 * _YLIM_FACTOR),
        "TTE-ylim": (0, 15 * _YLIM_FACTOR),
    }
    VIS_VALUES.update({
        "EN-yTicks": VIS_VALUES["EN-yLabels"],
        "EB-yTicks": [0, .25, .5, .75, 1],
        "T-yTicks": VIS_VALUES["T-yLabels"],
        "TTE-yTicks": VIS_VALUES["TTE-yLabels"],
    })
    # # After https://www.color-hex.com/color-palette/81979
    # BG_COLORS = {
    #     "BG1": "#72c4d0",
    #     "BG2": "#91d9d3",
    #     "BG3": "#afeed5",
    #     "BG4": "#54afcd"
    # }
    ERROR_WIDTH = 2.5
    ERROR_COLOR = "#000000"

    VIS_NUMERIC_ORDER = [
        QUESTION_TYPES[0],
        QUESTION_TYPES[2],
        QUESTION_TYPES[5]
    ]
    VIS_BOOL_ORDER = [
        QUESTION_TYPES[1],
        QUESTION_TYPES[3],
        QUESTION_TYPES[4],
        QUESTION_TYPES[6],
        QUESTION_TYPES[7],
    ]

    ERRORS_TITLE = "Errors split by LLAT"
    CORRECTNESS_TITLE = "Correctness split by LLAT"
    TIME_TITLE = "Completion time split by LLAT"
    ERROR_AXIS = ""
